from typing import Tuple, List
from inspect import getmembers

from .namespace import Namespace
from .fields.base import Field


class Schema:
    """Schema class. Inherit and define your own schemas"""

    @classmethod
    def _get_fields(cls) -> List[Tuple[str, Field]]:
        return getmembers(cls, predicate=lambda x: isinstance(x, Field))

    @classmethod
    def parse(cls, data: dict) -> Namespace:
        """Given a dictionary (data), parses and returns a Namespace containing attributes defined
        as Fields on the Schema.

        :param data: A dict or dict like object to parse
        :returns: A Namespace object
        """
        ns_kwargs = {}

        for key, field in cls._get_fields():
            raw_value = data.get(key)
            ns_kwargs[key] = field.validate(key, raw_value)

        return Namespace(**ns_kwargs)



from argparse import Namespace as _Namespace
from typing import Optional, Any, Sequence


class Namespace(_Namespace):
    """Simple namespace object returned when parsing a Schema"""

    def get(self, key: str, default: Optional[Any] = None) -> Any:
        """Get an attribute from the namespace, provide an optional default value"""
        return getattr(self, key, default)

    def to_dict(self, exclude: Optional[Sequence] = None) -> dict:
        """Returns the Namespace as a dictionary. An optional `exclude` param can be supplied
        to exclude values from the response.

        :param exclude: An optional sequence of values to exclude
        :returns: A dictionary of the Namespace keys and values
        """
        return {
            k: getattr(self, k)
            for k, v in vars(self).items()
            if k not in set(exclude or [])
        }



from .schema import Schema


def strict_types(cls: Schema):
    """strict_types decorator. Decorate a Schema instance to enforce `strict_type` on all Fields"""
    fields = cls._get_fields()
    for name, field in fields:
        field.strict_type = True
    return cls



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="liaison",
    packages=["liaison"],
    author="Julian Nash",
    version="0.1",
    description="Liaison is a Python library for defining schemas, parsing and validating payloads",
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords="python schema parser",
    url="https://github.com/Julian-Nash/liaison",
    project_urls={
        "Bug Tracker": "https://github.com/Julian-Nash/liaison",
        "Documentation": "https://github.com/Julian-Nash/liaison",
        "Source Code": "https://github.com/Julian-Nash/liaison",
    },
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    extras_require={
        "dev": ["pytest", "black", "coverage"],
        "test": ["pytest", "coverage"],
    }
)